<?php

declare(strict_types=1);


final class MysqliPreparedStatement
{
    /**
     * @var mysqli_stmt
     */
    private $stmt;

    /**
     * MysqliPreparedStatement constructor.
     *
     * @param mysqli_stmt $stmt
     */
    public function __construct(mysqli_stmt $stmt)
    {
        $this->stmt = $stmt;
    }

    public function get(array $params, int $resultType): MysqliResult
    {
        $this->execute($params);
        $mysqliResult = $this->stmt->get_result();
        if (!$mysqliResult) {
            $this->except("Get result failed: %s");
        }

        return new MysqliResult($mysqliResult, $resultType);
    }

    public function execute(array $params): void
    {
        $types = $this->mysqliParamTypes($params);

        $bindParam = $this->stmt->bind_param(implode($types), ...$params);

        if (!$bindParam) {
            $this->except("Binding parameters failed: %s");
        }

        $this->stmt->execute();
    }

    private function mysqliParamTypes(array $params): array
    {
        $typeMapping = [
            'boolean' => 'i',
            'integer' => 'i',
            'float' => 'd',
            'string' => 's',
            'null' => 's',
        ];

        return array_map(
            function ($param) use ($typeMapping): string {
                if (is_object($param) && method_exists($param, '__toString')) {
                    $param = (string)$param;
                }

                $type = mb_strtolower(gettype($param));

                if (!array_key_exists($typeMapping, $type)) {
                    throw new InvalidArgumentException(
                        sprintf(
                            'Param of type %s cannot be used in a query',
                            $type
                        )
                    );
                }

                return $typeMapping[$type];
            },
            $params
        );
    }

    private function except(string $format): void
    {
        throw new LogicException(
            sprintf(
                $format,
                $this->stmt->error
            ),
            $this->stmt->errno
        );
    }

    public function __destruct()
    {
        $this->stmt->close();
    }
}
